<?php
/**
 * NP_Twitter Plugin for NucleusCMS
 *
 * History
 * v0.1
 *   - initialize version
 * v0.2
 *   - improve update encoding
 *   - rename %%TWITTERD%% to %%TDATE%%
 *   - fix option label
 *   - twitthis support
 *   - fix GMT timestamp bug
 *   - tweet on new post
 * v0.3
 *   - added icon formatting override
 *   - fixed draft post tweet (which should not)
 *   - changed page=xx to tpage=
 *   - fix timezone bug 
 *   - tweetlet
 * v0.4
 *   - fix date under flow caused by < GMT timezone
 *   - handle time/date overflow caused by > GMT timezone
 *   - fix htmlspecialchars_decode support for PHP4
 *   - curl_setopt_array php4 support
 *   - no tweet on future post
 * v0.5
 *   - fixed future post tweet SQL check
 *   - fixed update tweet login check.
 *   - added title, excerpt in auto tweet (%t, %e)
 *   - added date sepecrator in tweets archive
 *   - handled protected updates
 *   - fixed &quot; and +
 *   - added delete tweet function
 *   - added reply function to add @xxx to the tweetbox (using javascript)
 *   - fixed URL mess up on reading:
 * v0.6
 *   - changed only when tweetbox appear that reply/delete buttons are shown
 *   - only allow delete own tweets
 *   - reply button to only friends
 *   - reply button only to those who follow me
 *   - char counter on update box 
 *   - fixed timezone error.... 10/0??? hopefully once and for all this time...
 *   - fixed random tweet on new post
 *   - added archive skinVar mode "mytweets" to show only a particular user's tweets
 *   - added char count to tweetlet
 *   - fixed curl warning
 *   - fix user option bug
 * v0.7
 *   - new post tweet send override
 * v0.8
 *   - direct message sending
 *   - added archive tweets override function
 *   - added client source nptwitter
 *   - allowed reply to all (since a twitterer might sett to show other ppl's replies)
 *   - highlight replies (search for @admun)
 *   - provided failure response when tweet/DM failed to sent
 *   - abort tweet delete, if delete from twitter timeout/failed
 *   - added twitter_all_update.php to download all missing tweets in the
 *     past (200 at a time)
 *   - fixed incorrect count update friends/follower
 *   - fixed & < > ' encoded display
 *   - fixed a bug when user has not using twitter
 *   - fixed & => &amp; encoding when sending update
 *   - added favorite support
 *   - direct msg download and display
 *   - show favorite tweets
 *   - reply dm
 *   - del dm
 *   - sync favorite w/ local tweets
 * V0.9
 *   - skip CURL returns for sendTweet
 *   - set VARCHAR size to 255 to support MySQL version < 5.0.3
 *   - add replyid to update so reply to someone is link properly
 *   - set fix icon image size
 *   - download replies (include those twitters I am not following), show all replies
 *   - properly decode specialchars
 *   - fixed for Twitpocalypse
 *

Note: The current implementation of dm, replies, favourites incured a slow query, it should store in its own table to improve performance....

admun TODO

- allow tweet on editing draft
- retweet function
- use JustPost event for 3.3 for tweet on future post
- search
- use tinyurl API (see http://remysharp.com/tinyurlapi) http://remysharp.com/tinyurlapi?callback=tinyurlCallback&url=_URL_
- emiticon support
- link to @who
- allow favorite on protected tweets, only unprotected is show when not login

- how long ago, using javascript to speed up?
- new replies notification
- fix change icon problem, maybe localize icon, or always use current icon 
- track unread tweets, highlight (via CSS), added "set all to read up to here"

- error handling when del tweet failure
- error handling by HTTP status code
- fix hicup on tweetlet on ' in title
javascript:location.href='http://edmondhui.homeip.net/blog/action.php?action=plugin&name=Twitter&op=tweetlet&url='+encodeURIComponent(location.href)+'&text='+document.title

- re-implement GUI using Ajax/jQuery
- use PHP wtitter library
- followings tag/group support

- multiple-tweet template, with admin menu
- language file
- spool tweets and resend later when send failure

- add current user tweet
- show badge for login member
- per member accounts... or via member options?
- daily tweets digest
- make XML parser classes
 */

if (!function_exists('curl_setopt_array')) {
   function curl_setopt_array(&$ch, $curl_options)
   {
       foreach ($curl_options as $option => $value) {
           if (!curl_setopt($ch, $option, $value)) {
               return false;
           }
       }
       return true;
   }
}

if ( !function_exists('htmlspecialchars_decode') )
{
    function htmlspecialchars_decode($text)
    {
        return strtr($text, array_flip(get_html_translation_table(HTML_SPECIALCHARS)));
    }
}

if (!function_exists('sql_table')) {
	function sql_table($name){
		return 'nucleus_'.$name;
	}
}

// include tweet parser
include($DIR_PLUGINS."/twitter/tweets_parser.php");

class NP_Twitter extends NucleusPlugin {
	function init() {
		$this->tweets_tab = sql_table('plug_twitters_tweets');
		$this->friends_tab = sql_table('plug_twitters_friends');
		$this->followers_tab = sql_table('plug_twitters_followers');
		$this->twitters_tab = sql_table('plug_twitters_info');
		$this->dm_tab = sql_table('plug_twitters_dm');
		$this->twitter_url = "http://twitter.com";
		$this->client = "NP_Twitter";
		$this->version = "0.9";
	}

	function getName() {
		return 'Twitter';
	}

	function getAuthor() {
		return 'Edmond Hui (admun)';
	}

	function getURL() {
		return 'http://forum.nucleuscms.org/';
	}

	function getVersion() {
		return $this->version;
	}

	function getDescription() {
		return 'This plugin provides Twitter integration';
	}

	function getEventList() {
		return array(
			'PostPluginOptionsUpdate',
			'PostDeleteMember',
			'AddItemFormExtras',
			'PostAddItem'
		);
	}

	function updateTwittersUser($memberid) {
		$session = curl_init();

		$user = $this->getMemberOption($memberid,'TwitterUser');
		$password = $this->getMemberOption($memberid,'TwitterPassword');

		if (!$user || !$password) return;

		$header = array("X-Twitter-Client: " . $this->client, 
                          "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => $this->twitter_url
			. '/users/'.$user.'.xml',
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_HTTPHEADER => $header,
                        CURLOPT_RETURNTRANSFER => 1
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		$result = curl_exec($session);
		curl_close($session);

		//echo "<!-- ". $result . "-->\n";

		$parser = xml_parser_create();
		xml_set_element_handler($parser, "startTag", "endTag");
		xml_set_character_data_handler($parser, "u_contents"); 

		if(!(xml_parse($parser, $result))){
			echo "<div id=\"content\">";
			echo "<div class=\"error\">";
			echo "NP_Twitter: Failed to update Twitter info, please try again";
			echo "</div>";
			echo "</div>";
			xml_parser_free($parser); 
			return;
		}

		xml_parser_free($parser); 

		global $uid;

		if (!$uid) {
			echo "<div id=\"content\">";
			echo "<div class=\"error\">";
			echo "NP_Twitter: Failed to update Twitter info, please try again";
			echo "</div>";
			echo "</div>";
			return;
		}

		sql_query('DELETE FROM ' . $this->twitters_tab . ' WHERE authorid='. $memberid);
		sql_query('INSERT INTO ' . $this->twitters_tab . ' (authorid,uid) VALUES (\''.$memberid.'\',\''.$uid.'\')');
	}

	function event_PostPluginOptionsUpdate($data) {
		// should update the member's twitters id in $this->twitters_tab
		if ($data['context']== 'member') {
			$this->updateTwittersUser($data['member']->getID());
		}
	}

	function event_PostDeleteMember(&$data) {
		$uid = getTwitterId($data['member']->getID());
		sql_query('DELETE FROM ' . $this->friends_tab . ' WHERE uid=' . $uid);
		sql_query('DELETE FROM ' . $this->followers_tab . ' WHERE uid=' . $uid);
		sql_query('DELETE FROM ' . $this->twitters_tab . ' WHERE authorid='. $data['member']->getID());
		sql_query('DELETE FROM ' . $this->dm_tab . ' WHERE rid='. $data['member']->getID());
	}

	/*
	function event_QuickMenu(&$data) {
		global $member;

		if (!($member->isLoggedIn())) return;

		array_push(
			$data['options'],
			array('title' => 'Twitter',
			'url' => $this->getAdminURL(),
			'tooltip' => 'Tweet tweet')
		);
	}
	*/

	function supportsFeature($f) {
		switch($f){
			case 'SqlTablePrefix':
				return 1;
			default:
				return 0;
		}
	}	

	/** 
	 * Creates the technoratitags table if it doesn't exist yet
	 */
	function install() {
		sql_query('CREATE TABLE IF NOT EXISTS '.$this->tweets_tab.' (`text` VARCHAR(160) NOT NULL, `created_at` DATETIME NOT NULL, `fid` INT NOT NULL, `tid` INT NOT NULL, `uid` INT NOT NULL, `sname` VARCHAR(15) NOT NULL, `name` VARCHAR(40) NOT NULL, `purl` VARCHAR(255), PRIMARY KEY (`tid`, `uid`, `fid`), FULLTEXT (`text`,`sname`,`name`))');
		sql_query('CREATE TABLE IF NOT EXISTS '.$this->friends_tab.' (`uid` INT NOT NULL, `fid` INT NOT NULL, `name` VARCHAR(40) NOT NULL, `sname` VARCHAR(15) NOT NULL, `loc` VARCHAR(30) NOT NULL, `descp` VARCHAR(160) NOT NULL, `url` VARCHAR(255) NOT NULL, `purl` VARCHAR(255) NOT NULL, PRIMARY KEY (`fid`), FULLTEXT(`name`))');
		sql_query('CREATE TABLE IF NOT EXISTS '.$this->followers_tab.' (`uid` INT NOT NULL, `fid` INT NOT NULL, `name` VARCHAR(40) NOT NULL, `sname` VARCHAR(15) NOT NULL, `loc` VARCHAR(30) NOT NULL, `descp` VARCHAR(160) NOT NULL, `url` VARCHAR(255) NOT NULL, `purl` VARCHAR(255) NOT NULL, PRIMARY KEY (`fid`), FULLTEXT(`name`))');
		sql_query('CREATE TABLE IF NOT EXISTS '.$this->twitters_tab.' (`authorid` TINYINT NOT NULL, `uid` INT NOT NULL, PRIMARY KEY (`authorid`, `uid`))');

		// added in V0.5
		sql_query('ALTER TABLE '.$this->friends_tab.' ADD `protected` BOOL NOT NULL AFTER `purl`');
		sql_query('ALTER TABLE '.$this->followers_tab.' ADD `protected` BOOL NOT NULL AFTER `purl`');
		sql_query('ALTER TABLE '.$this->tweets_tab.' ADD `protected` BOOL NOT NULL AFTER `purl`');

		// added in V0.8
		sql_query('ALTER TABLE '.$this->tweets_tab.' ADD `favorite` BOOL NOT NULL');
		sql_query('CREATE TABLE IF NOT EXISTS '.$this->dm_tab.' (`text` VARCHAR(160) NOT NULL, `created_at` DATETIME NOT NULL, `tid` INT NOT NULL, `sid` INT NOT NULL, `rid` INT NOT NULL, `sname` VARCHAR(40) NOT NULL, `rname` VARCHAR(40) NOT NULL, `purl` VARCHAR(255), PRIMARY KEY (`tid`, `sid`, `rid`), FULLTEXT (`text`,`sname`,`rname`))');

		// added in V0.9
		sql_query('ALTER TABLE '.$this->dm_tab.' CHANGE `tid` `tid` BIGINT( 11 ) NOT NULL');
		sql_query('ALTER TABLE '.$this->dm_tab.' CHANGE `rid` `rid` BIGINT( 11 ) NOT NULL');
		sql_query('ALTER TABLE '.$this->dm_tab.' CHANGE `sid` `sid` BIGINT( 11 ) NOT NULL');

		sql_query('ALTER TABLE '.$this->followers_tab.' CHANGE `uid` `uid` BIGINT( 11 ) NOT NULL');
		sql_query('ALTER TABLE '.$this->followers_tab.' CHANGE `fid` `fid` BIGINT( 11 ) NOT NULL');

		sql_query('ALTER TABLE '.$this->friends_tab.' CHANGE `uid` `uid` BIGINT( 11 ) NOT NULL');
		sql_query('ALTER TABLE '.$this->friends_tab.' CHANGE `fid` `fid` BIGINT( 11 ) NOT NULL');

		sql_query('ALTER TABLE '.$this->twitters_tab.' CHANGE `uid` `uid` BIGINT( 11 ) NOT NULL');

		sql_query('ALTER TABLE '.$this->tweets_tab.' CHANGE `tid` `tid` BIGINT( 11 ) NOT NULL');
		sql_query('ALTER TABLE '.$this->tweets_tab.' CHANGE `fid` `fid` BIGINT( 11 ) NOT NULL');
		sql_query('ALTER TABLE '.$this->tweets_tab.' CHANGE `uid` `uid` BIGINT( 11 ) NOT NULL');

		$this->createOption('Header','Tweets header formating','text','<ul>');
		$this->createOption('Item','Tweets formating (%%TWITTERT%% - twitter name in text, %%TWITTERI%% - twitter name as image, %%TWEET%% - tweet, %%TDATE%% - date)','text','<li>%%TWITTERT%%: %%TWEET%%</li>');
		$this->createOption('Footer','Tweets footer formating','text','</ul>');
		$this->createOption('TweetOnNewPost','Tweet on new item posted?','yesno','no');
		$this->createOption('NewPostTexts','Text to tweet on new post (one per line, will be randomly pick, %l == item url, %t == title, %e == excerpt)','textarea',"Just posted to my blog, see \"%t\" (%l)\nSee what I just wrote \"%t\" @ %l\nMy blog has updated \"%t\" (%l)\n");

		$this->createOption('Cleanup','Delete tweets cache on uninstall','yesno','no');
		$this->createOption('ShowDate','Show Date in Archive?','yesno','yes');

		$this->createMemberOption('TwitterUser','username','text','');
		$this->createMemberOption('TwitterPassword','password','password','');
	}
	
	function unInstall() {
		if ($this->getOption('Cleanup') == 'yes'){
			sql_query('DROP TABLE '.$this->tweets_tab);
		}

		sql_query('DROP TABLE '. $this->friends_tab);
		sql_query('DROP TABLE '. $this->followers_tab);
		sql_query('DROP TABLE '. $this->twitters_tab);
		sql_query('DROP TABLE '. $this->dm_tab);
	}

	/** 
	 * Returns array of tables to be additionally included in the 
	 * backup process
	 */
	function getTableList() {
		return array($this->tweets_tab,$this->friends_tab,$this->followers_tab,$this->twitters_tab);
	}

	function doTweets($uid, $num, $page=0, $icon='', $tweetType, $atweet='no') {
		global $CONF, $member;

		$loggedIn = $member->isLoggedIn();

		if (!$loggedIn && ($atweet == "dm" || $attweet == 'fav')) {
			echo "Access denied";
			return;
		}

		if ($loggedIn) {
			$authorid = $member->getID();
			$username = $this->getMemberOption($authorid,'TwitterUser');
		}

                if ($atweet == "yes") {
                        $tweetType = "tweets";
                }

		if ($uid == '') return;

		$start = ($page - 1) * $num;
		if ($start < 0) $start = 0;

		$extra = '';
		if (!$loggedIn) {
			$extra = ' AND t.protected != 1 ';
		}
		
		if ($tweetType == "mytweets") {
			$extra .= ' AND t.fid = ' . $uid . ' ';
		}

		//select t.text,t.tid,t.uid,t.sname,t.name,t.created_at,f.purl from nucleus_plug_twitters_tweets as t,nucleus_plug_twitters_friends as f where t.uid=4185371 and f.uid=t.fid order by t.tid desc limit 0,20;
		if ($atweet == 'dm') {
			$query = sql_query("SELECT * FROM " . $this->dm_tab . " as t WHERE t.rid=" . $uid . " ORDER BY created_at DESC LIMIT " . $start . "," . $num);
		}
		elseif ($atweet == 'fav') {
			$query = sql_query("SELECT * FROM " . $this->tweets_tab . " as t WHERE t.uid=" . $uid . $extra . " And t.favorite=1 ORDER BY created_at DESC LIMIT " . $start . "," . $num);
		}
		elseif ($atweet == 'rep') {
			$query = sql_query("SELECT * FROM " . $this->tweets_tab . " as t WHERE t.uid=" . $uid . $extra . " AND t.text LIKE \"%@". $username ."%\" ORDER BY created_at DESC LIMIT " . $start . "," . $num);
		} else {
			$query = sql_query("SELECT * FROM " . $this->tweets_tab . " as t WHERE t.uid=" . $uid . $extra . " ORDER BY created_at DESC LIMIT " . $start . "," . $num);
		}

		if (mysql_num_rows($query) == 0) return;

		$date_head = $this->getOption('ShowDate');

		if ($page == 0) $date_head = "no";

		echo "<div id=\"twitter\">";
		echo $this->getOption('Header');
		while ($row = mysql_fetch_object($query)) {
			$new_head = substr($row->created_at, 0, 10);
			if ($new_head != $date_head && $date_head != "no") {
				echo "<li><h3>".$new_head."</h3></li>";
				$date_head = $new_head;
			}

			$fromt = "<a href=\"" . $this->twitter_url . "/" 
			       . $row->sname . "\"><b>" . $row->sname .
			       "</b></a>";
			$fromi = "<a href=\"" . $this->twitter_url . "/" 
			       . $row->sname . "\"><img src=\"" 
			       . $row->purl . "\" alt=\"" . $row->sname 
			       . "\" title=\"" . $row->sname . "\" height=\"48\" width=\"48\" /></a>";
			$date = "<a href=\"" . $this->twitter_url . "/"
			      . $row->sname . "/statuses/" . $row->tid 
			      . "\">" . $row->created_at . "</a>";

			if ($loggedIn) {
				global $updateboxed;
				if ($updateboxed == "true" && $atweet != "dm") {
					if ($row->fid != $row->uid) {
						$date .= " <a href=\"javascript:insertReplyTo('@"
						. $row->sname . "','" . $row->tid
						. "')\">[reply]</a>";

						$date .= " <a href=\"javascript:insertDmTo('"
						. $row->sname
						. "')\">[directmail]</a>";
					} else {
						$date .= 
						" <a href=\"" . $CONF['ActionURL'] 
						. "?action=plugin&name=Twitter&op=deltweet&tid="
						. $row->tid . "&redirecturl=http://" 
						. serverVar("HTTP_HOST")
						. serverVar('REQUEST_URI')
						. "\">[delete]</a> "; 
					}

					if (!$row->favorite) {
						if ($row->protected == 0) {
							$date .= 
							" <a href=\"" . $CONF['ActionURL'] 
							. "?action=plugin&name=Twitter&op=favtweet&tid="
							. $row->tid . "&redirecturl=http://" 
							. serverVar("HTTP_HOST")
							. serverVar('REQUEST_URI')
							. "\">[favorite]</a> "; 
						}
					} else {
						$date .= 
						" <a href=\"" . $CONF['ActionURL'] 
						. "?action=plugin&name=Twitter&op=unfavtweet&tid="
						. $row->tid . "&redirecturl=http://" 
						. serverVar("HTTP_HOST")
						. serverVar('REQUEST_URI')
						. "\">[unfavorite]</a> "; 
					}

				}
				elseif ($updateboxed == "true" && $atweet == "dm") {
						$date .= " <a href=\"javascript:insertReplyTo('@"
						. $row->sname
						. "')\">[reply]</a>";

						$date .= " <a href=\"javascript:insertDmTo('"
						. $row->sname
						. "')\">[directmail]</a>";
						$date .= 
						" <a href=\"" . $CONF['ActionURL'] 
						. "?action=plugin&name=Twitter&op=deldm&tid="
						. $row->tid . "&redirecturl=http://" 
						. serverVar("HTTP_HOST")
						. serverVar('REQUEST_URI')
						. "\">[delete]</a> "; 
				}

                                // highlight replies to user
                                if (preg_match("/@". $username . "/", $row->text)) {
                                        $row->text = preg_replace('/@'. $username .'/', "<b class=\"myreply\">@". $username ."</b>", $row->text);
                                }

			}

			// shorten URL
			$msg = COMMENT::prepareBody($row->text);

			$tweet = $this->getOption('Item');
			// override the icon setting
			if ($icon == 'text') {
				$tweet = str_replace('%%TWITTERI%%','%%TWITTERT%%',$tweet);
			} else if ($icon == 'icon') {
				$tweet = str_replace('%%TWITTERT%%','%%TWITTERI%%',$tweet);
			}

                        // link to @someone
                        //$msg = preg_replace("/@.+ {0,1}/", "<a href=\"".$this->twitter_url."/"."aa"."/\">"."aa"."</a> ", $msg);

			$tweet = str_replace('%%TWITTERT%%',$fromt,$tweet);
			$tweet = str_replace('%%TWITTERI%%',$fromi,$tweet);
			$tweet = str_replace('%%TDATE%%',$date,$tweet);
			$tweet = str_replace('%%TWEET%%',$msg,$tweet);

			// somehow I'm seeing &amp;gt;  need decode twice?? once to change &amp; to &, then &gt; to >
			$tweet = htmlspecialchars_decode($tweet);
			$tweet = htmlspecialchars_decode($tweet);

			echo $tweet;

		}
		echo $this->getOption('Footer');
		
                $xtra = "";
                if ($atweet == "yes") {
                        $xtra = "&atweet=yes";
                }

		if (RequestVar('show') == "dm" || $atweet == "dm") {
			$xtra = "&show=dm";
		}
		if (RequestVar('show') == "fav" || $atweet == "fav") {
			$xtra = "&show=fav";
		}
		if (RequestVar('show') == "rep" || $atweet == "rep") {
			$xtra = "&show=rep";
		}

		if ($page > 1) {
			$pre = $page - 1;
			echo "<div class=\"prev\"><a href=\"?tpage=" . $pre 
			   . $xtra . "\">&lt;&lt; newer</a></div>";
		}

		if ($page >= 1 && mysql_num_rows($query) == $num) {
			$next = $page + 1;
			echo "<div class=\"next\"><a href=\"?tpage=" . $next
			   . $xtra . "\">older &gt;&gt;</a></div>";
		}
		echo "</div>";
	}

	function doScript($uid, $num, $page, $icon, $tweetType, $atweet) {
		global $CONF, $updateboxed;

                if ($atweet == '') {
                        $atweet = "no";
                }

		if ($updateboxed == "true") {
			$rep = '&rep=1';
		} else {
			$rep = '';
		}

		if ($tweetType == "tweets") {
			$updatetype = "update";
		} else {
			$updatetype = "aupdate";
		}

		?>
		  <!-- code from http://dutchcelt.nl/weblog/article/ajax_for_weblogs/ -->
		  <script type="text/javascript">
		  <!--
		    var ajaxTW=false;
		    /*@cc_on @*/
		    /*@if (@_jscript_version >= 5)
		    try {
			  ajaxTW = new ActiveXObject("Msxml2.XMLHTTP");
		    } catch (e) {
			  try {
			    ajaxTW = new ActiveXObject("Microsoft.XMLHTTP");
			  } catch (E) {
			    ajaxTW = false;
			  }
		    }
		    @end @*/
		 
		    if (!ajaxTW && typeof XMLHttpRequest!='undefined') {
			  ajaxTW = new XMLHttpRequest();
		    }
		 
		    function TWgetMyHTML() {
		 
			  var serverPage = '<?php echo $CONF['IndexURL']; ?>action.php?action=plugin&name=Twitter&op=<? echo $updatetype; ?>&uid=<?php echo $uid?>&num=<?php echo $num; ?>&tpage=<?php echo $page . $rep; ?>&icon=<? echo $icon; ?>&atweet=<? echo $atweet; ?>'; 
                          var objTW = document.getElementById('twitter');
			  ajaxTW.open("GET", serverPage);
			  ajaxTW.onreadystatechange = function() {
				if (ajaxTW.readyState == 4 && ajaxTW.status == 200) {
				  objTW.innerHTML = ajaxTW.responseText;
				}
			  }
			  ajaxTW.send(null);
		 
			TWstartRefresh();
		   }
		 
		   function TWstartRefresh() {
			 setTimeout("TWgetMyHTML()",3*60*1000);
		   }
		 
		   // trick learnt from wp wordspew
		   if(typeof window.addEventListener != 'undefined') {
			 //.. gecko, safari, konqueror and standard
			 window.addEventListener('load', TWstartRefresh, false);
		   }
		   else if(typeof document.addEventListener != 'undefined')
		   {
			 //.. opera 7
			 document.addEventListener('load', TWstartRefresh, false);
		   }
		   else if(typeof window.attachEvent != 'undefined')
		   {
			 //.. win/ie
			 window.attachEvent('onload', TWstartRefresh);
		   }
		  // -->
		  </script>
		<?
	}

	function doTwitthis($style) {
		$out = "";
		if ($style == "icon") {
			$out = "<!-- Begin TwitThis (http://twitthis.com/) -->
				<script type=\"text/javascript\" src=\"http://s3.chuug.com/chuug.twitthis.scripts/twitthis.js\"></script>
				<script type=\"text/javascript\">
				<!--
				document.write('<a href=\"javascript:;\" onclick=\"TwitThis.pop();\"><img src=\"http://s3.chuug.com/chuug.twitthis.resources/twitthis_grey_72x22.gif\" alt=\"TwitThis\" style=\"border:none;\" /></a>');
				//-->
				</script>
				<!-- /End -->";
		} elseif ($style== "text") {
			$out = "<!-- Begin TwitThis (http://twitthis.com/) -->
				<script type=\"text/javascript\" src=\"http://s3.chuug.com/chuug.twitthis.scripts/twitthis.js\"></script>
				<script type=\"text/javascript\">
				<!--
				document.write('<a href=\"javascript:;\" onclick=\"TwitThis.pop();\">TwitThis</a>');
				//-->
				</script>
				<!-- /End --> ";
		}

		echo $out;
	}

	function doTemplateVar(&$item, $type, $style = 'icon') {
		if ($type == "twitthis") {
			$this->doTwitthis($style);
		}
	}

	function doSkinVar($skinType, $type, $memb='', $num=0, $icon='') {

		if ($type == "updatebox") {
			global $updateboxed;
			$updateboxed = "true";
			global $CONF, $member;
			if (!$member->isLoggedIn()) { return; }

			// ED$ check if this user has configure his/her account 

			$this->addCharCountScript();
			echo "<form method=\"post\" action=\"".$CONF['ActionURL']."\">\n"
              			. "<div class=\"Twitter\">\n"
				. "<input type=\"hidden\" name=\"action\" value=\"plugin\" />\n"
				. "<input type=\"hidden\" name=\"name\" value=\"Twitter\" /> \n"
				. "<input type=\"hidden\" name=\"redirecturl\" value=\"" 
				. serverVar('REQUEST_URI') . "\" />\n"
				. "<input type=\"hidden\" name=\"op\" value=\"tweet\" />\n"
				. "<label for=\"tweet_box\">  </label>"
				. "<textarea name=\"text\" rows=\"2\" cols=\"70\" id=\"tweet_box\" onKeyUp=\"Contar('tweet_box','tweet_count','{CHAR} characters left.',140);\"/></textarea>\n"
				. "<input type=\"submit\" class=\"button\" value=\"Tweet!\" /><br/>"
				. "<input id=\"replyid_box\" type=\"hidden\" size=\"20\" name=\"replyto\" value=\"\" />\n"
                                . "send as direct msg to <input id=\"dm_box\" type=\"text\" size=\"12\" name=\"direct\"/><br/>"
				. "<span id=\"tweet_count\" class=\"minitext\">140 characters left.</span>"
				. "</div>"
				. "</form>\n";

                        // Warn when tweet failed to send
                        if (RequestVar('failure') == '1') {
                                echo "<b>Tweet sent failed</b><br/>";
                        }

			return;
		}

		if ($type == "linktodm") {
			if (RequestVar('show') != 'dm') {
				echo "[<a href=\"" . serverVar('SCRIPT_URI') . "?show=dm&tpage=1\">Show Directmail</a>]";
			}
			return;
		}

		if ($type == "linktofav") {
			if (RequestVar('show') != 'fav') {
				echo "[<a href=\"" . serverVar('SCRIPT_URI') . "?show=fav&tpage=1\">Show Favorite</a>]";
			}
			return;
		}

		if ($type == "linktorep") {
			if (RequestVar('show') != 'rep') {
				echo "[<a href=\"" . serverVar('SCRIPT_URI') . "?show=rep&tpage=1\">Show Replies</a>]";
			}
			return;
		}


		if ($type == "twitthis") {
			$style = memb; // this parameter is overlay with membername
			$this->doTwitthis($style);
		}

                if ($type == "linktoatweet") {
                        $atweet = RequestVar('atweet');
                        if ($atweet == "yes") {
                                return;
                        }

                        echo "<br/><a href=\"" . serverVar('SCRIPT_URI') . "?tpage=1&atweet=yes\">Wanna see tweets from all my followers?</a>";
                        return;
                }

		$page = intRequestVar('tpage');

		$mem = MEMBER::createFromName($memb);
		$authorid = $mem->getID();
		$uid = $this->getTwitterId($authorid);
		$username = $this->getMemberOption($authorid,'TwitterUser');

		if ($type == "tweets" || $type == "mytweets") {
		        $atweet = RequestVar('atweet');
			$show = RequestVar('show');
			if ($show == 'dm') {
				$atweet = 'dm';
			}

			if ($show == 'fav') {
				$atweet = 'fav';
			}

			if ($show == 'rep') {
				$atweet = 'rep';
			}

			$this->doScript($uid, $num, $page, $icon, $type, $atweet);
			$this->doTweets($uid, $num, $page, $icon, $type, $atweet);
		}

		if ($type == "friends") {
			$query = sql_query("SELECT * FROM " 
			       . $this->friends_tab . " WHERE uid=" . $uid);
			if (mysql_num_rows($query) == 0) return;

			while ($row = mysql_fetch_object($query)) {
				echo "<img src=\"" . $row->purl . "\" alt=\""
				   . $row->sname . "\" title=\"" . $row->sname
				   . "\"/>";
			}
		}

		if ($type == "link") {
			$label = $num;
			echo "<a href=\"" . $this->twitter_url . "/" 
			   . $username . "\">" . $label . "</a>";
		}

		if ($type == "stats") {
			$query = sql_query("SELECT * FROM " 
			. $this->friends_tab . " WHERE uid=" . $uid);
			echo "following <a href=\"" . $this->twitter_url . "/" 
			. $username . "/friends\">" .  mysql_num_rows($query) 
			. " people</a><br/>";
			$query = sql_query("SELECT * FROM " 
			. $this->followers_tab . " WHERE uid=" . $uid);
			echo mysql_num_rows($query) . " people following me<br/>";
		}
	}

	function getTwitterId($memberid) {
		if (!intval($memberid)) return 0;
		$result = sql_query('SELECT uid FROM ' . $this->twitters_tab 
		        . ' WHERE authorid=' . $memberid);
		$author = mysql_fetch_object($result);
		return $author->uid;
	}

	function updateDM($user, $password, $memberid) {
		global $parsed_arr, $counter, $current_tag;
		$uid = $this->getTwitterId($memberid);
		$offset = date('Z'); // get timezone offset (in sec)

		if ($uid == '') return;

		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client,
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => $this->twitter_url
			. '/direct_messages.xml',
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_HTTPHEADER => $header,
                        CURLOPT_RETURNTRANSFER => 1
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		$result = curl_exec($session);
		curl_close($session);

		// echo "<!-- ". $result . "-->\n";

		$parser = xml_parser_create();
		xml_set_element_handler($parser, "startTag", "endTag");
		xml_set_character_data_handler($parser, "dm_contents"); 

		if(!(xml_parse($parser, $result))){
		    die("Error on line " . xml_get_current_line_number($parser));
		}

		for ($x=1;$status = array_pop($parsed_arr);$x++) {
			$gmt_time = strtotime($status->created);
			$created_at = date("Y-n-j H:i:s", $gmt_time);

			echo $x . ": " . $status->text . " ("
			. $status->sid .  " (" . $status->sname . "), " . $status->rid 
			. " (" . $status->rname . "), " . $created_at . " [" . $status->created . "]<br/>\n";

			sql_query('INSERT INTO ' . $this->dm_tab 
				. ' (text,created_at,tid,sid,rid,sname,rname,purl) VALUES (\''
				. htmlspecialchars($status->text, ENT_QUOTES) 
				. '\',\''
				. $created_at . '\',\'' 
				. $status->tid . '\',\'' . $status->sid . '\',\''
				. $status->rid . '\',\'' 
				. htmlspecialchars($status->sname, ENT_QUOTES)
				. '\',\'' . htmlspecialchars($status->rname, ENT_QUOTES) 
				. '\',\'' . $status->purl
				. '\') ON DUPLICATE KEY UPDATE tid=tid'
			);
		}

		$counter = 0;
		$current_tag = "";
		xml_parser_free($parser); 
	}

	function updateFriendsTL($user, $password, $memberid) {
		global $parsed_arr, $counter, $current_tag;
		$uid = $this->getTwitterId($memberid);
		$offset = date('Z'); // get timezone offset (in sec)

		if ($uid == '') return;

		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client,
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => $this->twitter_url
			. '/statuses/friends_timeline.xml',
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_HTTPHEADER => $header,
                        CURLOPT_RETURNTRANSFER => 1
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		// for update
		//curl_setopt ( $session, CURLOPT_POSTFIELDS,"status=" . $status);

		$result = curl_exec($session);
		curl_close($session);

		//echo "<!-- ". $result . "-->\n";

		$parser = xml_parser_create();
		xml_set_element_handler($parser, "startTag", "endTag");
		xml_set_character_data_handler($parser, "ft_contents"); 

		if(!(xml_parse($parser, $result))){
		    die("Error on line " . xml_get_current_line_number($parser));
		}

		for ($x=1;$status = array_pop($parsed_arr);$x++) {
			$gmt_time = strtotime($status->created);
			$created_at = date("Y-n-j H:i:s", $gmt_time);

			echo $x . ": " . $status->text . " ("
			. $status->screen_name .  ", " . $status->name 
			. ", " . $created_at . " [" . $status->created . "], "
			. $status->protected . ")<br/>\n";

			sql_query('INSERT INTO ' . $this->tweets_tab 
				. ' (text,created_at,tid,uid,fid,sname,name,purl,protected) VALUES (\''
				. htmlspecialchars($status->text, ENT_QUOTES) 
				. '\',\''
				. $created_at . '\',\'' 
				. $status->tid . '\',\'' . $uid . '\',\''
				. $status->uid . '\',\'' 
				. htmlspecialchars($status->screen_name, ENT_QUOTES)
				. '\',\'' . htmlspecialchars($status->name, ENT_QUOTES) 
				. '\',\'' . $status->purl
				. '\',\'' . $status->protected
				. '\') ON DUPLICATE KEY UPDATE uid=uid'
			);
		}

		$counter = 0;
		$current_tag = "";
		xml_parser_free($parser); 
	}
	
	function updateFav($user, $password, $memberid) {
		global $parsed_arr, $counter, $current_tag;
		$uid = $this->getTwitterId($memberid);
		$offset = date('Z'); // get timezone offset (in sec)

		if ($uid == '') return;

		// Yes, I am talking a short-cut here, should be getting all the favorites in loop, instead of only the recent 20.....

		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client,
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => $this->twitter_url
			. '/favorites.xml',
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_HTTPHEADER => $header,
                        CURLOPT_RETURNTRANSFER => 1
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		// for update
		//curl_setopt ( $session, CURLOPT_POSTFIELDS,"status=" . $status);

		$result = curl_exec($session);
		curl_close($session);

		//echo "<!-- ". $result . "-->\n";

		$parser = xml_parser_create();
		xml_set_element_handler($parser, "startTag", "endTag");
		xml_set_character_data_handler($parser, "ft_contents"); 

		if(!(xml_parse($parser, $result))){
		    die("Error on line " . xml_get_current_line_number($parser));
		}

		for ($x=1;$status = array_pop($parsed_arr);$x++) {
			$gmt_time = strtotime($status->created);
			$created_at = date("Y-n-j H:i:s", $gmt_time);

			echo $x . ": " . $status->text . " ("
			. $status->screen_name .  ", " . $status->name 
			. ", " . $created_at . " [" . $status->created . "], "
			. $status->protected . ")<br/>\n";

			sql_query('UPDATE ' . $this->tweets_tab . ' SET favorite=1 WHERE tid=' . $status->tid);
		}

		$counter = 0;
		$current_tag = "";
		xml_parser_free($parser); 
	}
	
	function updateRep($user, $password, $memberid) {
		global $parsed_arr, $counter, $current_tag;
		$uid = $this->getTwitterId($memberid);
		$offset = date('Z'); // get timezone offset (in sec)

		if ($uid == '') return;

		// Yes, I am talking a short-cut here, should be getting all the favorites in loop, instead of only the recent 20.....

		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client,
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => $this->twitter_url
			. '/statuses/replies.xml',
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_HTTPHEADER => $header,
                        CURLOPT_RETURNTRANSFER => 1
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		// for update
		//curl_setopt ( $session, CURLOPT_POSTFIELDS,"status=" . $status);

		$result = curl_exec($session);
		curl_close($session);

		//echo "<!-- ". $result . "-->\n";

		$parser = xml_parser_create();
		xml_set_element_handler($parser, "startTag", "endTag");
		xml_set_character_data_handler($parser, "ft_contents"); 

		if(!(xml_parse($parser, $result))){
		    die("Error on line " . xml_get_current_line_number($parser));
		}

		for ($x=1;$status = array_pop($parsed_arr);$x++) {
			$gmt_time = strtotime($status->created);
			$created_at = date("Y-n-j H:i:s", $gmt_time);

			echo $x . ": " . $status->text . " ("
			. $status->screen_name .  ", " . $status->name 
			. ", " . $created_at . " [" . $status->created . "], "
			. $status->protected . ")<br/>\n";

			sql_query('INSERT INTO ' . $this->tweets_tab 
				. ' (text,created_at,tid,uid,fid,sname,name,purl,protected) VALUES (\''
				. htmlspecialchars($status->text, ENT_QUOTES) 
				. '\',\''
				. $created_at . '\',\'' 
				. $status->tid . '\',\'' . $uid . '\',\''
				. $status->uid . '\',\'' 
				. htmlspecialchars($status->screen_name, ENT_QUOTES)
				. '\',\'' . htmlspecialchars($status->name, ENT_QUOTES) 
				. '\',\'' . $status->purl
				. '\',\'' . $status->protected
				. '\') ON DUPLICATE KEY UPDATE uid=uid'
			);
		}

		$counter = 0;
		$current_tag = "";
		xml_parser_free($parser); 
	}
	
	function updateFollowers($user, $password, $memberid) {
		global $parsed_arr, $counter, $current_tag;
		$uid = $this->getTwitterId($memberid);

		if ($uid == '') return;

		sql_query('DELETE FROM ' . $this->followers_tab . ' WHERE uid=' . $uid);

                $page = 1;
                $x = 100;

                do {
			$session = curl_init();

			$header = array("X-Twitter-Client: ". $this->client,
					"X-Twitter-Client-Version: ". $this->version);
			$options = array(
				CURLOPT_URL => $this->twitter_url . '/statuses/followers.xml?page=' . $page,
				CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
				CURLOPT_HEADER => 0,
				CURLOPT_USERPWD => "$user:$password",
				CURLOPT_HTTPHEADER => $header,
				CURLOPT_RETURNTRANSFER => 1
			);

			//print_r($options);
			curl_setopt_array($session, $options);

			$result = curl_exec($session);
			curl_close($session);

			//echo "<!-- ". $result . "-->\n";

			$parser = xml_parser_create();
			xml_set_element_handler($parser, "startTag", "endTag");
			xml_set_character_data_handler($parser, "fl_contents"); 

			if(!(xml_parse($parser, $result))){
			    die("Error on line " . xml_get_current_line_number($parser));
			}

			for ($x=1;$status = array_pop($parsed_arr);$x++) {
				echo $x . ": " . $status->name . " ("
				. $status->id . ", " . $status->sname . ", " 
				. $status->loc . ", " . $status->desc . ", "
				. $status->url . ", " . $status->protected 
				.  ")<br/>\n";

				if ($status->protected == 'true') {
					$status->protected = 1;
				} else {
					$status->protected = 0;
				}

				sql_query('REPLACE ' . $this->followers_tab . ' (uid,fid,name,loc,descp,url,purl,protected) VALUES (\'' . $uid . '\',\'' . $status->id . '\',\'' . htmlspecialchars($status->name, ENT_QUOTES) . '\',\'' . htmlspecialchars($status->loc, ENT_QUOTES) . '\',\'' . htmlspecialchars($status->desc, ENT_QUOTES) . '\',\'' . $status->url . '\',\'' . $status->purl . '\',\'' . $status->protected . '\')');
			}

			$page++;
                } while ($x >=99);

		$counter = 0;
		$current_tag = "";
		xml_parser_free($parser); 
	}

	function updateFriends($user, $password, $memberid) {
		global $parsed_arr, $counter, $current_tag;
		$uid = $this->getTwitterId($memberid);

		if ($uid == '') return; 

		sql_query('DELETE FROM ' . $this->friends_tab . ' WHERE uid=' . $uid);

                $page = 1;
                $x = 100;

                do {
		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client,
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => $this->twitter_url . '/statuses/friends.xml?page=' . $page,
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_HTTPHEADER => $header,
                        CURLOPT_RETURNTRANSFER => 1
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		$result = curl_exec($session);
		curl_close($session);

		//echo "<!-- ". $result . "-->\n";

		$parser = xml_parser_create();
		xml_set_element_handler($parser, "startTag", "endTag");
		xml_set_character_data_handler($parser, "fd_contents"); 

		if(!(xml_parse($parser, $result))){
		    die("Error on line " . xml_get_current_line_number($parser));
		}

		for ($x=1;$status = array_pop($parsed_arr);$x++) {
			echo $x . ": " . $status->name . " ("
			. $status->id . ", " . $status->sname . ", " 
			. $status->loc . ", " . $status->desc . ", " 
			. $status->url . ", " . $status->protected 
			. ")<br/>\n";

			if ($status->protected == 'true') {
				$status->protected = 1;
			} else {
				$status->protected = 0;
			}

			sql_query('REPLACE ' . $this->friends_tab . ' (uid,fid,name,sname,loc,descp,url,purl,protected) VALUES (\'' . $uid . '\',\'' . $status->id . '\',\'' . htmlspecialchars($status->name, ENT_QUOTES) . '\',\'' . htmlspecialchars($status->sname, ENT_QUOTES) . '\',\'' . htmlspecialchars($status->loc, ENT_QUOTES) . '\',\'' . htmlspecialchars($status->desc, ENT_QUOTES) . '\',\'' . $status->url . '\',\'' . $status->purl . '\',\'' . $status->protected . '\')');
		}

                $page++;
                } while ($x >=99);
                // We loop until twitter return less then 99 friends, which mean we are at the
                // last page of friends info

		$counter = 0;
		$current_tag = "";
		xml_parser_free($parser); 
	}

	function delTweet($user, $password, $tid, $type) {
		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client,
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => 
			$this->twitter_url . '/' . $type . '/destroy/' . $tid 
			. ".xml",
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_POST => 1,
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_HTTPHEADER => $header
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		$result = curl_exec($session);
		curl_close($session);

                ACTIONLOG::add(INFO, "tweet deleted: " . $tid);

		// ED$ - should check whether tweet is deleted b4 delete local copy
		if ($type == "statuses") {
	        	sql_query('DELETE FROM ' . $this->tweets_tab . ' WHERE tid='. $tid);
		} else {
	        	sql_query('DELETE FROM ' . $this->dm_tab . ' WHERE tid='. $tid);
		}
	}

	function favTweet($user, $password, $tid, $op) {
		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client,
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => 
			$this->twitter_url . '/favorites/' . $op . '/' . $tid
			. ".xml",
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_POST => 1,
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_HTTPHEADER => $header
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		curl_exec($session);
		curl_close($session);

		// ED$ - should check whether operation success b4 tag the fav tweet...
		if ($op == "create") {
			$v = 1;
		} else {
			$v = 0;
		}
		sql_query("UPDATE " . $this->tweets_tab . " SET favorite=" . $v . " WHERE tid=" . $tid);
	}

	function sendTweet($user, $password, $text, $replyid=0) {

		$text = str_replace('&','&amp;',$text);
		if ($replyid > 0) {
			$reply = '&in_reply_to_status_id=' . $replyid;
		}

		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client,
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => $this->twitter_url 
			. '/statuses/update.xml',
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_POST => 1,
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_HTTPHEADER => $header,
			CURLOPT_POSTFIELDS => "status=" 
			. urlencode($text) . "&source=nptwitter" . $reply
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		$result = curl_exec($session);
		curl_close($session);

                //ACTIONLOG::add(INFO, "Tweet sent: " . $result);
                return $result;
	}

	function sendDirectTweet($user, $password, $to, $text) {
		$session = curl_init();

		$header = array("X-Twitter-Client: " . $this->client, 
                                "X-Twitter-Client-Version: " . $this->version);
		$options = array(
			CURLOPT_URL => $this->twitter_url 
			. '/direct_messages/new.xml',
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_HEADER => 0,
			CURLOPT_USERPWD => "$user:$password",
			CURLOPT_POST => 1,
			CURLOPT_HTTPHEADER => $header,
			CURLOPT_POSTFIELDS => "user=$to&text=" 
			. urlencode($text) . "&source=nptwitter"
		);

		//print_r($options);
		curl_setopt_array($session, $options);

		$result = curl_exec($session);
		curl_close($session);

                ACTIONLOG::add(INFO, "Direct tweet sent: " . $result);
                return $result;
	}

	function doAction($type) {
		$action = RequestVar('op');

		if ($action == "refresh") {
			$query = sql_query("SELECT mnumber FROM " . sql_table('member'));
			while ($row = mysql_fetch_object($query)) {
				// for each, user show, refresh friends, refresh followers

				$user = $this->getMemberOption($row->mnumber,'TwitterUser');
				$password = $this->getMemberOption($row->mnumber,'TwitterPassword');

				if (!$user || !$password) continue;

				echo "<h2>Updating friends...</h2>\n";
				$this->updateFriends($user,$password,$row->mnumber);
				echo "<h2>Updating followers...</h2>\n";
				$this->updateFollowers($user,$password,$row->mnumber);
				echo "<h2>Updating tweets...</h2>\n";
				$this->updateFriendsTL($user,$password,$row->mnumber);
				echo "<h2>Getting DM...</h2>\n";
				$this->updateDM($user,$password,$row->mnumber);
				echo "<h2>Updating favorites...</h2>\n";
				$this->updateFav($user,$password,$row->mnumber);
				echo "<h2>Updating replies...</h2>\n";
				$this->updateRep($user,$password,$row->mnumber);
			}

		}

		// For testing only
		if ($action == "rrr") {
			$query = sql_query("SELECT mnumber FROM " . sql_table('member'));
			while ($row = mysql_fetch_object($query)) {
				$user = $this->getMemberOption($row->mnumber,'TwitterUser');
				$password = $this->getMemberOption($row->mnumber,'TwitterPassword');

				$this->updateRep($user,$password,$row->mnumber);
			}
		}

		if ($action == "update" || $action == "aupdate") {
			$uid = RequestVar('uid');
			$num = RequestVar('num');
			$page = RequestVar('tpage');
			$icon = RequestVar('icon');
                        $atweet = RequestVar('atweet');

			if (IntRequestVar('rep') == 1) {
				global $updateboxed;
				$updateboxed = "true";
			}

			if ($action == "update") {
				$tweetype = "tweets";
			} else {
				$tweetype = "mytweets";
			}

                        //ACTIONLOG::add(INFO, 'NP_Twitter: Ajax update trigger (action=' . $action . ')');

			$this->doTweets($uid, $num, $page, $icon, $tweetype, $atweet);
		}

		if ($action == "tweet") {
			global $member;
			if (!$member->isLoggedIn()) doError('You\'re not logged in.');
			$redirect=RequestVar('redirecturl');
                        // strip failure indicator from last tweet action
                        $redirect=str_replace("&failure=1", "", $redirect);

			$text = RequestVar('text');
			$direct = RequestVar('direct');
			$text = substr($text,0,140);
			$tweetas =  RequestVar('tweetas');
			$replyid =  RequestVar('replyto');

			if ($replyid != '') {
				ACTIONLOG::add(INFO, "replyid: " . $replyid);
			}

			if ($tweetas != '') {
				$text = $tweetas . " " . $text;
			}

			$user = $this->getMemberOption($member->getID(),'TwitterUser');
			$password = $this->getMemberOption($member->getID(),'TwitterPassword');

			if (!$user || !$password) return;


                        if ($direct == "") {
      			        $r = $this->sendTweet($user, $password, $text, $replyid);
                        } else {
      			        $r = $this->sendDirectTweet($user, $password, $direct, $text);
                        }

                        if (!$r) {
                                $redirect .= "&failure=1";
                        }

			header('Location: ' . $redirect);
		}

		if ($action == "deltweet") {
			global $member;
			if (!$member->isLoggedIn()) doError('You\'re not logged in.');
			$tid =  RequestVar('tid');
			$redirect = RequestVar('redirecturl');

			$user = $this->getMemberOption($member->getID(),'TwitterUser');
			$password = $this->getMemberOption($member->getID(),'TwitterPassword');

			if ($user && $password) {
      				$this->delTweet($user, $password, $tid, "statuses");
			}

			header('Location: ' . $redirect);
		}

		if ($action == "deldm") {
			global $member;
			if (!$member->isLoggedIn()) doError('You\'re not logged in.');
			$tid =  RequestVar('tid');
			$redirect = RequestVar('redirecturl');

			$user = $this->getMemberOption($member->getID(),'TwitterUser');
			$password = $this->getMemberOption($member->getID(),'TwitterPassword');

			if ($user && $password) {
      				$this->delTweet($user, $password, $tid, "direct_messages");
			}

			header('Location: ' . $redirect);
		}

		if ($action == "favtweet" || $action == "unfavtweet") {
			global $member;
			if (!$member->isLoggedIn()) doError('You\'re not logged in.');
			$tid =  RequestVar('tid');
			$redirect = RequestVar('redirecturl');

			$user = $this->getMemberOption($member->getID(),'TwitterUser');
			$password = $this->getMemberOption($member->getID(),'TwitterPassword');

			if ($action == "favtweet") {
				$op = "create";
			} else {
				$op = "destroy";
			}

			if ($user && $password) {
      				$this->favTweet($user, $password, $tid, $op);
			}

			header('Location: ' . $redirect);
		}

		if ($action == "tweetlet") {
			global $member;
			if (!$member->isLoggedIn()) doError('You\'re not logged in.');
			global $CONF;
			$url =  RequestVar('url');

			$this->addCharCountScript();
			echo "<h1>Tweetlet</h1><form method=\"post\" action=\""
			.$CONF['ActionURL']."\">\n" 
			. "<div class=\"Twitter\">\n"
			. "<input type=\"hidden\" name=\"action\" value=\"plugin\" />\n"
			. "<input type=\"hidden\" name=\"name\" value=\"Twitter\" />\n"
			. "<input type=\"hidden\" name=\"redirecturl\" value=\""
			. $url . "\" />\n" 
			. "<input type=\"hidden\" name=\"op\" value=\"tweet\" />\n"
			. "<select name=\"tweetas\">
				<option selected>Reading:
				<option>Looking at:
				<option>Listening to:
				<option>Laughing at:
				<option>at:
				<option>Waiting for:
				<option>Looking forward to:
				</select><br/>"
			. "<textarea name=\"text\" rows=\"2\" cols=\"70\" id=\"tweet_box\" onKeyUp=\"Contar('tweet_box','tweet_count','{CHAR} characters left.',120);\"/>\n"
			. RequestVar('text') ." ". RequestVar('url') 
			. "</textarea>\n"
			. "<br><span id=\"tweet_count\" class=\"minitext\">" 
			. intVal(120 - strlen(RequestVar('text') ." ". RequestVar('url'))) 
			. " characters left.</span><br/>"
			. "<input type=\"submit\" class=\"button\" value=\"Tweet!\" />"
			. "</div>"
			. "</form>\n";
		}
	}

	function event_AddItemFormExtras($data) {
?>
		<h3>Twitter</h3>
		<p>
                <input type="checkbox" name="tweet_this_post"
<?php
                if ($this->getOption('TweetOnNewPost') == "yes") {
                        echo " checked=\"checked\"";
                }
?>
                />Send tweet on this post
                </p>
<?php
	}

	function event_PostAddItem($data) {
		$tweet_this = RequestVar('tweet_this_post');
		if ($tweet_this == "on") {
			$itemid = $data['itemid'];
			$query=sql_query('SELECT iauthor FROM ' . sql_table('item') . ' WHERE inumber=' . $itemid . ' AND idraft=0 AND itime <= NOW()');
			if (mysql_num_rows($query) == 0) return;

			$row=mysql_fetch_object($query);
			$authorId=$row->iauthor;

			$user = $this->getMemberOption($authorId,'TwitterUser');
			$password = $this->getMemberOption($authorId,'TwitterPassword');

			if (!$user || !$password) {
				return;
			}

			$pre_arr = explode("\n", $this->getOption('NewPostTexts'));
			if (count($pre_arr) > 1) {
				$text = $pre_arr[rand(0,count($pre_arr)-1)];
			} else {
				$text = $pre_arr[0];
			}

			global $manager;
			$item = &$manager->getItem($itemid, 0, 0);

	        	$url = createItemLink($itemid);
			$text = str_replace("%l",$url,$text);

			// ED$ only do this if %t is used, else we can used 110 words
			$exc_to_see = 60; // 110-strlen($item['title']);

			$text = str_replace("%t",$item['title'],$text);
			$text = str_replace("%e", strip_tags(substr($item['body'], 0, $exc_to_see)) . "...", $text);

			ACTIONLOG::add(INFO, 'Tweet on new post: ' . $text);
      			$result = $this->sendTweet($user, $password, $text);
		}
	}

	function addCharCountScript()
	{
?>
<script type="text/javascript">
  function insertReplyTo(text, text2){
    document.getElementById('replyid_box').value= text2;
    document.getElementById('tweet_box').value+= text + " ";
    document.getElementById('tweet_box').focus();
  }
  function insertDmTo(text){
    document.getElementById('dm_box').value+= text;
    document.getElementById('dm_box').focus();
  }
</script>
<script type="text/javascript">
<!-- code taken from http://javascript.internet.com/forms/character-counter.html -->

function getObject(obj) {
  var theObj;
  if(document.all) {
    if(typeof obj=="string") {
      return document.all(obj);
    } else {
      return obj.style;
    }
  }
  if(document.getElementById) {
    if(typeof obj=="string") {
      return document.getElementById(obj);
    } else {
      return obj.style;
    }
  }
  return null;
}

//Contador de caracteres.
function Contar(entrada,salida,texto,caracteres) {
  var entradaObj=getObject(entrada);
  var salidaObj=getObject(salida);
  var longitud=caracteres - entradaObj.value.length;
  if(longitud <= 0) {
    longitud=0;
    texto='<span class="disable"> '+texto+' </span>';
    entradaObj.value=entradaObj.value.substr(0,caracteres);
  }
  salidaObj.innerHTML = texto.replace("{CHAR}",longitud);
}
</script>
<?
	}

}
?>
